package	jp.co.sds_corp.util.pooh.dialog;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.pooh.top.Toplevel;
import jp.co.sds_corp.util.prop.CmnPropParser;
import jp.co.sds_corp.util.prop.ELM;
import jp.co.sds_corp.util.xml.CmnXMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
public class DialogParser {

	private Toplevel toplevel = null;
	public static final String BUTTON_PANE= "jp.co.sds_corp.util.pooh.simple.SimpleButtonPane";
	public DialogParser(Toplevel _toplevel) { toplevel = _toplevel; }
	public DialogResource loadResource(String fn) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn);
			return parser.getDialogResource();
		} else {
			LoadProperty parser = new LoadProperty(fn);
			return parser.getDialogResource();
		}
	} 
	public DialogResource loadResource(String fn, String path) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getDialogResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getDialogResource();
		}
	} 
	public DialogResource loadResource(String fn, String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getDialogResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getDialogResource();
		}
	} 

	public DialogResource [] loadResourceAll(String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(null, path);
			return parser.getDialogResources();
		} else {
			LoadProperty parser = new LoadProperty(null, path);
			return parser.getDialogResources();
		}
	} 

	private class LoadProperty extends CmnPropParser {
		private DialogResource dialogResource = null;
		private List list = new ArrayList();
		private String fn;
		public LoadProperty(String _fn) throws Exception {
			fn = _fn;
			parse(parseFile(toplevel.getResourcePath() + ".FRAME-INF.dialog")); 
		}

		public LoadProperty(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(_path));
		}

		public void parse(ELM root) throws Exception {
			if (root == null) {
				throw new Exception("not tag <dialogs>");
			} else {
				boolean match = false;
				ELM [] e = getSpecificELMsByELM(root, "dialogs");
				if (e.length <= 0)
					return;
				ELM [] c = getSpecificELMsByELM(e[0], "dialog");
				for (int i=0; i<c.length; i++) {
					String id = getAttributeByELM(c[i], "DialogID");
					if (fn != null) {
						if (id.equals(fn)) {
							dialogResource = new DialogResource();
							dialogResource.setDialog_id(id);
							dialogDetail(c[i]);
							list.add(dialogResource);
							return;
						}
					} else {
						dialogResource = new DialogResource();
						dialogResource.setDialog_id(id);
						dialogDetail(c[i]);
						list.add(dialogResource);
						match = true;
					}
				}
				if (!match) { throw new Exception("<DialogID> : keys not found." + fn); }
			}
		}

		public DialogResource getDialogResource() { return dialogResource; }
		public DialogResource [] getDialogResources() { 
			DialogResource [] result = new DialogResource[list.size()];
			return (DialogResource [])list.toArray(result); 
		}
		protected void dialogDetail(ELM e) {
			dialogResource.setDialog_name(getAttributeByELM(e, "DialogName"));
			dialogResource.setDialog_type(getAttributeByELM(e, "DialogType"));
			dialogResource.setDialog_x(getAttributeByELM(e, "DialogX"));
			dialogResource.setDialog_y(getAttributeByELM(e, "DialogY"));
			dialogResource.setDialog_w(getAttributeByELM(e, "DialogSizeW"));
			dialogResource.setDialog_h(getAttributeByELM(e, "DialogSizeH"));
			dialogResource.setDialog_c(getAttributeByELM(e, "DialogClosed"));
			dialogResource.setDialog_style(getAttributeByELM(e, "DialogStyle"));
			dialogResource.setDialog_p(getAttributeByELM(e, "Panel"));
			dialogResource.setDialog_resize(getAttributeByELM(e, "DialogResize"));
			dialogResource.setDialog_btn(getAttributeByELM(e, "DialogBTN"));
			dialogResource.setDialog_btns(getAttributeByELM(e, "DialogButtons"));
			if (dialogResource.getDialog_btn().equals("custom")) {
				dialogResource.setDialog_btnp(getAttributeByELM(e, "ButtonPane"));
			} else {
				dialogResource.setDialog_btnp(BUTTON_PANE);
			}
		}

	}

	public class LoadXML extends CmnXMLParser {
		private DialogResource dialogResource = null;
		private List list = new ArrayList();
		private String fn;
		public LoadXML(String _fn) throws Exception {
			fn = _fn;
			URL url = toplevel.getResource("FRAME-INF.dialog", "xml");
			String charset = toplevel.getPoohCharset();
			parse(parse(url.openStream()));
		}
		public LoadXML(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(new File(_path)));
		}

		public void parse(Document doc) throws Exception {
			if (doc == null) {
				throw new Exception("not tag <dialogs>");
			} else {
				boolean match = false;
				Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "dialogs");
				if (e.length <= 0)
					return;
				Element [] c = getSpecificElementsByNode(e[0], "dialog");
				for (int i=0; i<c.length; i++) {
					String id = getAttributeByElement(c[i], "DialogID");
					if (fn != null) {
						if (id.equals(fn)) {
							dialogResource = new DialogResource();
							dialogResource.setDialog_id(id);
							dialogDetail(c[i]);
							list.add(dialogResource);
							return;
						}
					} else {
						dialogResource = new DialogResource();
						dialogResource.setDialog_id(id);
						dialogDetail(c[i]);
						list.add(dialogResource);
						match = true;
					}
				}
				if (!match) { throw new Exception("<DialogID> : keys not found." + fn); }
			}
		}
		public DialogResource getDialogResource() { return dialogResource; }
		public DialogResource [] getDialogResources() { 
			DialogResource [] result = new DialogResource[list.size()];
			return (DialogResource [])list.toArray(result); 
		}
		private void dialogDetail(Element e) {
			dialogResource.setDialog_name(getAttributeByElement(e, "DialogName"));
			dialogResource.setDialog_type(getAttributeByElement(e, "DialogType"));
			dialogResource.setDialog_x(getAttributeByElement(e, "DialogX"));
			dialogResource.setDialog_y(getAttributeByElement(e, "DialogY"));
			dialogResource.setDialog_w(getAttributeByElement(e, "DialogSizeW"));
			dialogResource.setDialog_h(getAttributeByElement(e, "DialogSizeH"));
			dialogResource.setDialog_c(getAttributeByElement(e, "DialogClosed"));
			dialogResource.setDialog_style(getAttributeByElement(e, "DialogStyle"));
			dialogResource.setDialog_p(getAttributeByElement(e, "Panel"));
			dialogResource.setDialog_resize(getAttributeByElement(e, "DialogResize"));
			dialogResource.setDialog_btn(getAttributeByElement(e, "DialogBTN"));
			dialogResource.setDialog_btns(getAttributeByElement(e, "DialogButtons"));
			if (dialogResource.getDialog_btn().equals("custom")) {
				dialogResource.setDialog_btnp(getAttributeByElement(e, "ButtonPane"));
			} else {
				dialogResource.setDialog_btnp(BUTTON_PANE);
			}
		}
	}
}
