package	jp.co.sds_corp.util.pooh.menu;

import java.io.File;
import java.net.URL;
import java.util.LinkedList;

import jp.co.sds_corp.util.pooh.top.Toplevel;
import jp.co.sds_corp.util.prop.CmnPropParser;
import jp.co.sds_corp.util.prop.ELM;
import jp.co.sds_corp.util.xml.CmnXMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class PopupParser {
	Toplevel toplevel;
	public static final String DEFAULT_ACTION ="jp.co.sds_corp.util.pooh.action.ActDefault";
	public PopupParser(Toplevel _toplevel) {toplevel = _toplevel;}

	public RootMenuResource loadResource(String fn) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn);
			return parser.getMenuResource();
		} else {
			LoadProperty parser = new LoadProperty(fn);
			return parser.getMenuResource();
		}
	}

	public RootMenuResource loadResource(String fn, String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getMenuResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getMenuResource();
		}
	}

	public PopupResource loadResourceAll(String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(null, path);
			return parser.getPopupResource();
		} else {
			LoadProperty parser = new LoadProperty(null, path);
			return parser.getPopupResource();
		}
	}

	private class LoadProperty extends CmnPropParser {
		private String fn;
		private RootMenuResource menuResource = null;
		private PopupResource popupResource = null;
		public LoadProperty(String _fn) throws Exception {
			fn = _fn;
			parse(parseFile(toplevel.getResourcePath() + ".FRAME-INF.popup"));
		}

		public LoadProperty(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(_path));
		}

		public void parse(ELM root) throws Exception {
			if (root == null) {
				throw new Exception("not tag <menus>");
			} else {
				boolean match = false;
				ELM [] e = getSpecificELMsByELM(root, "menus");
				for (int i=0; i<e.length; i++) {
					ELM [] c = getSpecificELMsByELM(e[i], "menu");
					for (int j=0; j<c.length; j++) {
						String name = getAttributeByELM(c[j], "name");
						if (fn != null) {
							if (name.equals(fn)) {
								menuResource = new RootMenuResource();
								menuResource.setName(name);
								checkSub(menuResource, menuResource.getName(), c[j]);
								return;
							}
						} else {
							if (popupResource == null)
								popupResource = new PopupResource();

							menuResource = new RootMenuResource();
							menuResource.setName(name);
							checkSub(menuResource, menuResource.getName(), c[j]);
							popupResource.addRootMenu(menuResource);
							match = true;
						}
					}
				}
			}
		}
		public RootMenuResource getMenuResource() { return menuResource; }
		public PopupResource getPopupResource() { return popupResource; }
		private void checkSub(RootMenuResource root, String p, ELM element) throws Exception {
			ELM [] e = getSpecificELMsByELM(element, "item");
			LinkedList sublist = null;
			if (e.length > 0) { sublist = new LinkedList(); }
			for (int i=0; i<e.length; i++) {
				SubMenuResource sub = new SubMenuResource(); 
				sub.setName(getAttributeByELM(e[i], "name"));
            	sub.setType(getAttributeByELM(e[i], "type"));
            	sub.setMnemonic(getAttributeByELM(e[i], "mnemonic"));
            	sub.setEnabled(getAttributeByELM(e[i], "enable"));
            	sub.setSelected(getAttributeByELM(e[i], "selected"));
            	sub.setListener(toplevel, getAttributeByELM(e[i], "listener"));
            	sub.setAction(getAttributeByELM(e[i], "action"));
            	sub.setOpen(getAttributeByELM(e[i], "open"));
            	sub.setLink(getAttributeByELM(e[i], "link"));
				sub.setAccelerator(getAttributeByELM(e[i], "accelerator"));

				sub.setRoot_name(root.getName());
				sub.setRoot_index(root.getMenu_index());
				sub.setParent_name(p);

				sub.setRootMenuResource(root);
				sublist.add(sub);
				root.addSubMenu(p, sublist);
            	checkSub(root, sub.getName(), e[i]);
        	}
		}
	}

	private class LoadXML extends CmnXMLParser {
		private String fn;
		private RootMenuResource menuResource = null;
		private PopupResource popupResource = null;
		public LoadXML(String _fn) throws Exception {
			URL url = toplevel.getResource("FRAME-INF.popup", "xml");
			String charset = toplevel.getPoohCharset();

			fn = _fn;
			parse(parse(url.openStream()));
		}

		public LoadXML(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(new File(_path)));
		}

		public void parse(Document doc) throws Exception {
			if (doc == null) {
				throw new Exception("not tag <menus>");
			} else {
				boolean match = false;
				Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "menus");
				for (int i=0; i<e.length; i++) {
					Element [] c = getSpecificElementsByNode(e[i], "menu");
					for (int j=0; j<c.length; j++) {
						String name = getAttributeByElement(c[j], "name");
						if (fn != null) {
							if (name.equals(fn)) {
								menuResource = new RootMenuResource();
								menuResource.setName(name);
								checkSub(menuResource, menuResource.getName(), c[j]);
								return;
							}
						} else {
							if (popupResource == null)
								popupResource = new PopupResource();

							menuResource = new RootMenuResource();
							menuResource.setName(name);
							checkSub(menuResource, menuResource.getName(), c[j]);
							popupResource.addRootMenu(menuResource);
							match = true;
						}
					}
				}
			}
		}

		public PopupResource getPopupResource() { return popupResource; }
		public RootMenuResource getMenuResource() { return menuResource; }
		private void checkSub(RootMenuResource root, String p, Element element) throws Exception {
			Element [] e = getSpecificElementsByNode(element, "item");
			LinkedList sublist = null;
			if (e.length > 0) { sublist = new LinkedList(); }
			for (int i=0; i<e.length; i++) {
				SubMenuResource sub = new SubMenuResource(); 
				sub.setName(getAttributeByElement(e[i], "name"));
            	sub.setType(getAttributeByElement(e[i], "type"));
            	sub.setMnemonic(getAttributeByElement(e[i], "mnemonic"));
            	sub.setEnabled(getAttributeByElement(e[i], "enable"));
            	sub.setSelected(getAttributeByElement(e[i], "selected"));
            	sub.setListener(toplevel, getAttributeByElement(e[i], "listener"));
            	sub.setAction(getAttributeByElement(e[i], "action"));
            	sub.setOpen(getAttributeByElement(e[i], "open"));
            	sub.setLink(getAttributeByElement(e[i], "link"));
				sub.setAccelerator(getAttributeByElement(e[i], "accelerator"));

				sub.setRoot_name(root.getName());
				sub.setRoot_index(root.getMenu_index());
				sub.setParent_name(p);

				sub.setRootMenuResource(root);
				sublist.add(sub);
				root.addSubMenu(p, sublist);
            	checkSub(root, sub.getName(), e[i]);
        	}
		}
	}
}
