package	jp.co.sds_corp.util.pooh.simple;

import java.awt.Component;
import java.awt.event.MouseEvent;

import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import jp.co.sds_corp.util.pooh.frame.FrameResource;
import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.pane.ChildPane;
import jp.co.sds_corp.util.pooh.popup.IPopup;
import jp.co.sds_corp.util.pooh.popup.IPopupMenuAction;
import jp.co.sds_corp.util.pooh.top.Toplevel;

public class SimpleInternalFrame extends JInternalFrame implements IPopup {
	private Toplevel toplevel = null;
	private FrameResource frameResource = null;
	private SimplePopup popup = null;

	private boolean popupActionEnabled = true;
	public SimpleInternalFrame(Toplevel _toplevel, FrameResource res) throws Exception {
		super(res.getFrame_name(), true, true, true, true);
		setBounds(_toplevel.getPreferredFrameRectangle(res.getFrame_x(), res.getFrame_y(),
							res.getFrame_w(), res.getFrame_h()));
		frameResource = res;
		toplevel = _toplevel;
		ChildPane childPane = null;
		if (res.getPanel() != null) { 
			childPane = (ChildPane)res.getPanel();
			((SimplePanel)childPane).setParent(toplevel, this, res);
			boolean ff = childPane.createPane();
			getContentPane().add((JPanel)childPane);
			if (ff && res.getFrame_popup()) {
				setupPopupComponent((SimplePanel)childPane);
			}
		}

		setClosable(res.getFrame_c());
		setMaximizable(res.getFrame_m());
		setIconifiable(res.getFrame_i());
		setResizable(res.getFrame_r());

		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent e) {
				closedByClick();
			}
		});
	}

	private void setupPopupComponent(SimplePanel p) {
		popup = new SimplePopup(this, frameResource);
		if (p != null) {
			Component [] components = p.getComponents();
			for (int i=0; i<components.length; i++) {
				popup.addPopupListener(components[i]);
			}
			popup.addPopupListener(p);
		}
	}

	public String getFrame_id() { return frameResource.getFrame_id(); }
	public void popupUse() {
		if (frameResource.getFrame_popup())
			setupPopupComponent((SimplePanel)getChildPane());
	}

	public void addPanel(JPanel p) {
		if (frameResource.getFrame_popup()) {
			setupPopupComponent((SimplePanel)p);
		}
		getContentPane().add((JPanel)p);
	}

	public void remove() {
		((SimplePanel)frameResource.getPanel()).removeAll();
		getContentPane().removeAll();
	}

	public void closed() {
		if (frameResource.getPanel() != null) {
			((ChildPane)frameResource.getPanel()).closed();
		}
	}

	public void setPopupComponent(Component comp) {}
	public ChildPane getChildPane() {return (ChildPane)frameResource.getPanel();}
	private void closedByClick() { 
		closed();
		String key = getFrame_id() 
				   + "@"
				   + getClass().getName()
				   + "@" 
				   + Integer.toHexString(hashCode());
		toplevel.getFrameManager().remove(key);
	}

	public void setPopupActionEnabled(boolean b) { popupActionEnabled = b; }
	//----------------------------------------------------------------------
	// Popup menu interface
	//----------------------------------------------------------------------
	public void doPopupMenuSelected(Object m) {
		try {
			if (m instanceof JMenuItem) {
			} else {
				if (popupActionEnabled) {
					Object listener = ((SubMenuResource)m).getListener();
					if (listener != null) {
						if (((SubMenuResource)m).getListener_type() 
									== SubMenuResource.LISTENER_INSTANCE) {
							((IPopupMenuAction)listener).setParent(toplevel);
							((IPopupMenuAction)listener).doAction();
						} else {
							toplevel.invoke(listener, (SubMenuResource)m);
						}
					}				
				}
			}
		} catch (Exception e) {}
	}

	public void doPopupMenuVisible(){}
	public Component getComponent() { return this; }
	public Object [][] getPopupMenuItem() {return null; }
	public boolean isShowOK(MouseEvent e) { return true; }
}
