package jp.co.sds_corp.util.pooh.simple;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.util.Iterator;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

import jp.co.sds_corp.util.pooh.menu.IMenuAction;
import jp.co.sds_corp.util.pooh.menu.MenuResource;
import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.top.Toplevel;

public class SimpleMenuBar extends JMenuBar {
	MenuResource menuResource = null;
	Toplevel toplevel = null;

	boolean listener = true;
	public SimpleMenuBar(Toplevel _toplevel, MenuResource _resource) {
		menuResource   = _resource;
		toplevel = _toplevel;

		for (int i=0; i<menuResource.getMenuCount(); i++) {
			RootMenuResource root = menuResource.getRootMenuResource(i);
			SimpleMenu simpleMenu = new SimpleMenu(root);
			subMenus(root.getName(), simpleMenu, null, root);
			add(simpleMenu);
		}
	}
	public void closed() {}

	public void setListenerEnabled(boolean b) { listener = b; }
	public void subMenus(String key, SimpleMenu _menu, ButtonGroup _group, RootMenuResource _root) {
		Iterator iterator = _root.iterator(key);
		if (iterator != null) {
			while (iterator.hasNext()) {
				SubMenuResource info = (SubMenuResource)iterator.next();
				switch (info.getType()) {
					case SubMenuResource.MENU_NORMAL:
						JMenuItem menuItem;
						if (info.getIcon() == null) {
							menuItem = new JMenuItem(info.getName(), info.getMnemonic());
						} else {
							menuItem = new JMenuItem(info.getName(), info.getIcon());
							menuItem.setMnemonic(info.getMnemonic());
						}
						menuItem.setEnabled(info.getEnabled());
						if (info.getAccelerator()) {
							menuItem.setAccelerator(
								KeyStroke.getKeyStroke(info.getMnemonic(), InputEvent.CTRL_MASK));
						}
						_menu.add(menuItem);
						menuItem.addActionListener(_menu);
						info.setThis(menuItem);
						info.setSimpleMenu(_menu);
						break;

					case SubMenuResource.MENU_SEPARATOR:
						_menu.addSeparator();
						info.setSimpleMenu(_menu);
						break;

					case SubMenuResource.MENU_MENU:
						SimpleMenu m1 = new SimpleMenu(info, _root);
						subMenus(info.getName(), m1, null, _root);
						info.setSimpleMenu(m1);
						_menu.add(m1);
						break;

					case SubMenuResource.MENU_GROUP:
						ButtonGroup group = new ButtonGroup();
						subMenus(info.getName(), _menu, group, _root);
						info.setThis(group);
						info.setSimpleMenu(_menu);
						break;

					case SubMenuResource.MENU_CHECK:
						JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem(info.getName());
						cbMenuItem.setSelected(info.getSelected());
						cbMenuItem.setMnemonic(info.getMnemonic());
						cbMenuItem.setEnabled(info.getEnabled());
						cbMenuItem.addActionListener(_menu);	
						_menu.add(cbMenuItem);
						info.setThis(cbMenuItem);
						info.setSimpleMenu(_menu);
						break;

					case SubMenuResource.MENU_RADIO:
						JRadioButtonMenuItem rbMenuItem = new JRadioButtonMenuItem(info.getName());
						rbMenuItem.setSelected(info.getSelected());
						rbMenuItem.setMnemonic(info.getMnemonic());
						rbMenuItem.setEnabled(info.getEnabled());
						rbMenuItem.addActionListener(_menu);	
						if (_group != null) { _group.add(rbMenuItem); }
						_menu.add(rbMenuItem);
						info.setThis(rbMenuItem);
						info.setSimpleMenu(_menu);
						break;
				}
			}
		}
	}

    //-----------------------------------------------------------------
    //  Menu Items.
    //-----------------------------------------------------------------
	public class SimpleMenu extends JMenu implements ActionListener {
		private RootMenuResource root = null;
		private JMenuItem menuItem = null;
		private String name;

		public SimpleMenu(RootMenuResource _root) {
			super(_root.getName());
			name = _root.getName();
			setMnemonic(_root.getMnemonic());
			setEnabled(_root.getEnabled());
			root = _root;
		}

		public SimpleMenu(SubMenuResource _sub, RootMenuResource _root) {
			super(_sub.getName());
			name = _sub.getName();
			setMnemonic(_sub.getMnemonic());
			setEnabled(_sub.getEnabled());
			root = _root;
		}

		public void actionPerformed(ActionEvent ae) {
			if (!listener) return;
			final SubMenuResource sub =
					root.getSubMenuResource(root.getName(), ((JMenuItem)ae.getSource()).getText());
			final Object action = sub.getListener();
			if (action != null) {
				if (sub.getListener_type() == SubMenuResource.LISTENER_INSTANCE) {
					for (int i=0; i<getItemCount(); i++) {
						if (ae.getSource() == getItem(i)) {
							((IMenuAction)action).setParent(toplevel);
							((IMenuAction)action).doAction(sub, ae, i);
							break;
						}
					}
				} else {
					try {
						toplevel.setAttribute("CURRENT_MENU_ACTION", sub);
						toplevel.invoke(action, sub);
					} catch (Exception e) {e.printStackTrace();}
				}
			}
		}
	}
}
