package	jp.co.sds_corp.util.pooh.toolbar;

import java.io.File;
import java.net.URL;

import jp.co.sds_corp.util.pooh.top.Toplevel;
import jp.co.sds_corp.util.prop.CmnPropParser;
import jp.co.sds_corp.util.prop.ELM;
import jp.co.sds_corp.util.xml.CmnXMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ToolbarParser {
	Toplevel toplevel;
	public static final String DEFAULT_ACTION ="jp.co.sds_corp.util.pooh.action.ActToolbarDefault";
	public ToolbarParser(Toplevel _toplevel) {toplevel = _toplevel;}

	public ToolbarResource loadResource(String fn) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn);
			return parser.getToolbarResource();
		} else {
			LoadProperty parser = new LoadProperty(fn);
			return parser.getToolbarResource();
		}
	}

	public ToolbarResource loadResource(String fn, String path) throws Exception {
		String p = toplevel.getPoohResource();
		if (p == null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getToolbarResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getToolbarResource();
		}
	}

	public ToolbarResource loadResource(String fn, String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(fn, path);
			return parser.getToolbarResource();
		} else {
			LoadProperty parser = new LoadProperty(fn, path);
			return parser.getToolbarResource();
		}
	}

	public ToolbarManager loadResourceAll(String path, String resType) throws Exception {
		if (resType == null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(null, path);
			return parser.getToolbarManager();
		} else {
			LoadProperty parser = new LoadProperty(null, path);
			return parser.getToolbarManager();
		}
	}

	private class LoadProperty extends CmnPropParser {
		private ToolbarResource menuResource = null;
		private ToolbarManager tManager = null;
		private String fn;
		public LoadProperty(String _fn) throws Exception {
			fn = _fn;
			parse(parseFile(toplevel.getResourcePath() + ".FRAME-INF.toolbar"));
		}

		public LoadProperty(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(_path));
		}

		public void parse(ELM root) throws Exception {
			if (root == null) {
				throw new Exception("not tag <menus>");
			} else {
				ELM [] e = getSpecificELMsByELM(root, "toolbars");
				for (int i=0; i<e.length; i++) {
					ELM [] c = getSpecificELMsByELM(e[i], "toolbar");
					for (int j=0; j<c.length; j++) {
						String name = getAttributeByELM(c[j], "name");
						if (fn != null) {
							if (name.equals(fn)) {
								menuResource = new ToolbarResource();
								menuResource.setToolbar_name(name);
								doRootMenu(c[j]);
								return;
							}
						} else {
							if (tManager == null) {
								tManager = new ToolbarManager();
							}
							menuResource = new ToolbarResource();
							menuResource.setToolbar_name(name);
							doRootMenu(c[j]);
							tManager.addToolbarResource(menuResource);
						}
					}
				}
			}
		}
		public ToolbarResource getToolbarResource() { return menuResource; }
		public ToolbarManager getToolbarManager() { return tManager; } 
		private void doRootMenu(ELM element) throws Exception {
			ELM [] e = getSpecificELMsByELM(element, "item");
			for (int i=0; i<e.length; i++) {
				ButtonInfo rootMenu = new ButtonInfo();
				rootMenu.setToolbar_name(menuResource.getToolbar_name());
				rootMenu.setName(getAttributeByELM(e[i],      "name"));
				rootMenu.setNameShow(getAttributeByELM(e[i],  "nameshow"));
				rootMenu.setBtnType(getAttributeByELM(e[i],   "type"));
				rootMenu.setEnabled(getAttributeByELM(e[i],   "enable"));
				rootMenu.setTip(getAttributeByELM(e[i],   "tip"));

				rootMenu.setRollIcon(getAttributeByELM(e[i],      "roll-icon"));
				rootMenu.setUpIcon(getAttributeByELM(e[i],        "up-icon"));
				rootMenu.setPressedIcon(getAttributeByELM(e[i],   "press-icon"));
				rootMenu.setDisabledIcon(getAttributeByELM(e[i],   "disable-icon"));
				rootMenu.setListener(getAttributeByELM(e[i], "listener"));
				rootMenu.setMenuLink(getAttributeByELM(e[i], "menu-link"));
				rootMenu.setBtn_index(i);
				menuResource.addButtonInfo(rootMenu);
			}
		}
	}

	private class LoadXML extends CmnXMLParser {
		private ToolbarResource menuResource = null;
		private ToolbarManager tManager = null;
		private String fn;

		public LoadXML(String _fn) throws Exception {
			URL url = toplevel.getResource("FRAME-INF.toolbar", "xml");
			String charset = toplevel.getPoohCharset();

			fn = _fn;
			parse(parse(url.openStream()));
		}

		public LoadXML(String _fn, String _path) throws Exception {
			fn = _fn;
			parse(parseFile(new File(_path)));
		}

		public void parse(Document doc) throws Exception {
			if (doc == null) {
				throw new Exception("not tag <menus>");
			} else {
				Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "toolbars");
				for (int i=0; i<e.length; i++) {
					Element [] c = getSpecificElementsByNode(e[i], "toolbar");
					for (int j=0; j<c.length; j++) {
						String name = getAttributeByElement(c[j], "name");
						if (fn != null) {
							if (name.equals(fn)) {
								menuResource = new ToolbarResource();
								menuResource.setToolbar_name(name);
								doRootMenu(c[j]);
								return;
							}
						} else {
							if (tManager == null) {
								tManager = new ToolbarManager();
							}

							menuResource = new ToolbarResource();
							menuResource.setToolbar_name(name);
							doRootMenu(e[i]);
							tManager.addToolbarResource(menuResource);
						}
					}
				}
			}
		}
		public ToolbarResource getToolbarResource() { return menuResource; }
		public ToolbarManager getToolbarManager() { return tManager; }

		private void doRootMenu(Element element) throws Exception {
			Element [] e = getSpecificElementsByNode(element, "item");
			for (int i=0; i<e.length; i++) {
				ButtonInfo rootMenu = new ButtonInfo();
				rootMenu.setToolbar_name(menuResource.getToolbar_name());
				rootMenu.setName(getAttributeByElement(e[i],      "name"));
				rootMenu.setNameShow(getAttributeByElement(e[i], "showname"));
				rootMenu.setBtnType(getAttributeByElement(e[i],   "type"));
				rootMenu.setEnabled(getAttributeByElement(e[i],   "enable"));
				rootMenu.setTip(getAttributeByElement(e[i],   "tip"));

				rootMenu.setRollIcon(getAttributeByElement(e[i],      "roll-icon"));
				rootMenu.setUpIcon(getAttributeByElement(e[i],        "up-icon"));
				rootMenu.setPressedIcon(getAttributeByElement(e[i],   "press-icon"));
				rootMenu.setDisabledIcon(getAttributeByElement(e[i],   "disable-icon"));
				rootMenu.setListener(getAttributeByElement(e[i], "listener"));
				rootMenu.setMenuLink(getAttributeByElement(e[i], "menu-link"));
				rootMenu.setBtn_index(i);
				menuResource.addButtonInfo(rootMenu);
			}
		}

	}
}
