package	jp.co.sds_corp.util.pooh.top;

import java.io.File;
import java.net.URL;

import jp.co.sds_corp.util.prop.CmnPropParser;
import jp.co.sds_corp.util.prop.ELM;
import jp.co.sds_corp.util.xml.CmnXMLParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class DesktopParser {

	private String resourcePath = null;
	private Toplevel toplevel;

	public DesktopParser(Toplevel _toplevel, String _resourcePath) {
		toplevel = _toplevel;
		resourcePath = _resourcePath;
	}
	public DesktopResource loadResource() throws Exception {
		String p = toplevel.getPoohResource();
		if (p==null || p.length() <= 0 || p.equals("xml")) {
			LoadXML parser = new LoadXML();
			return parser.getDesktopResource();
		} else {
			LoadProperty parser = new LoadProperty();
			return parser.getDesktopResource();
		}
	}

	public DesktopResource loadResource(String path, String resType) throws Exception {
		if (resType==null || resType.length() <= 0 || resType.equals("xml")) {
			LoadXML parser = new LoadXML(path);
			return parser.getDesktopResource();
		} else {
			LoadProperty parser = new LoadProperty(path);
			return parser.getDesktopResource();
		}
	}

	private class LoadProperty extends CmnPropParser {
		private DesktopResource resource = null;
		public LoadProperty() throws Exception { 
			parse(parseFile(resourcePath + ".FRAME-INF.desktop"));
		}

		public LoadProperty(String _path) throws Exception { 
			parse(parseFile(_path));
		}

		public DesktopResource getDesktopResource() { return resource; }
		private void parse(ELM root) throws Exception { 
   	    	if (root != null) {
				resource = new DesktopResource();

				ELM [] e = getSpecificELMsByELM(root, "desktop-attri");
   	        	resource.setStart_object(getAttributeByELM(e[0], "object"));
   	        	resource.setStart_object_name(getAttributeByELM(e[0], "name"));

				e = getSpecificELMsByELM(root, "products");
				if (e.length > 0) {
					resource.getProducts().setVersion(getAttributeByELM(e[0], "version"));
					resource.getProducts().setUrl(getAttributeByELM(e[0], "url"));
					resource.getProducts().setCopyright(getAttributeByELM(e[0], "copyright"));
					resource.getProducts().setProduct(getAttributeByELM(e[0], "product"));
				}
			}
			doDesktopParts(root);
			doLoadObject(root);
			doMenuChange(root);
		}

		private void doDesktopParts(ELM doc) throws Exception {
			ELM [] e = getSpecificELMsByELM(doc, "desktop-parts");
			for (int i=0;i<e.length;i++) {
				ELM [] f = getSpecificELMsByELM(e[i], "parts");
				for (int j=0;j<f.length;j++) {
					DesktopPartsInfo info = new DesktopPartsInfo();
					info.setType(getAttributeByELM(f[j], "type"));
					info.setResource(getAttributeByELM(f[j], "resource"));
					info.setObject(getAttributeByELM(f[j], "object"));
					info.setKey(getAttributeByELM(f[j], "key"));

					if (info.getInt_type() == DesktopPartsInfo.STATUS_BAR)
						info.setBarcount(getAttributeByELM(f[j], "barcount"));

					resource.addDesktop(info);
				}
        	}
		}

		private void doLoadObject(ELM doc) throws Exception {
			ELM [] e = getSpecificELMsByELM(doc, "load-object");
			for (int i=0;i<e.length;i++) {
				LoadObjectInfo info = new LoadObjectInfo();
				info.setName(getAttributeByELM(e[i], "name"));
				info.setObject_string(getAttributeByELM(e[i], "object"));
				resource.addLoadObject(info);
			}
		}

		private void doMenuChange(ELM doc) throws Exception {
			ELM [] e = getSpecificELMsByELM(doc, "menu-change");
			for (int i=0;i<e.length;i++) {
				ELM [] f = getSpecificELMsByELM(e[i], "logic");
				for (int j=0;j<f.length;j++) {
					MenuChangeInfo info = new MenuChangeInfo();
					info.setAttribute(getAttributeByELM(f[j], "attribute"));
					info.setProperty(getAttributeByELM(f[j], "property"));
					info.setAction(getAttributeByELM(f[j], "action"));
					info.setValue(getAttributeByELM(f[j], "value"));
					info.setKey(getAttributeByELM(f[j], "key"));
					resource.addMenu(info);
				}
			}
		}
	}

	private class LoadXML extends CmnXMLParser {
		private DesktopResource resource = null;
		public LoadXML() throws Exception {
			URL url = toplevel.getResource("FRAME-INF.desktop", "xml");
			String charset = toplevel.getPoohCharset();
   	    	parse(parse(url.openStream()));
		}

		public LoadXML(String _path) throws Exception {
			parse(parseFile(new File(_path)));
		}

		private void parse(Document doc) throws Exception {
   	    	if (doc != null) {
				resource = new DesktopResource();

   	        	Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "desktop-attri");
   	        	resource.setStart_object(getAttributeByElement(e[0], "object"));
   	        	resource.setStart_object_name(getAttributeByElement(e[0], "name"));

				e = getSpecificElementsByNode(doc.getDocumentElement(), "products");
				if (e.length > 0) {
					resource.getProducts().setVersion(getAttributeByElement(e[0], "version"));
					resource.getProducts().setUrl(getAttributeByElement(e[0], "url"));
					resource.getProducts().setCopyright(getAttributeByElement(e[0], "copyright"));
					resource.getProducts().setProduct(getAttributeByElement(e[0], "product"));
				}
			}
	
			doDesktopParts(doc);
			doLoadObject(doc);
			doMenuChange(doc);
    	}

		public DesktopResource getDesktopResource() { return resource; }

		private void doDesktopParts(Document doc) throws Exception {
			Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "desktop-parts");
			for (int i=0;i<e.length;i++) {
				Element [] f = getSpecificElementsByNode(e[i], "parts");
				for (int j=0;j<f.length;j++) {
					DesktopPartsInfo info = new DesktopPartsInfo();
					info.setType(getAttributeByElement(f[j], "type"));
					info.setResource(getAttributeByElement(f[j], "resource"));
					info.setObject(getAttributeByElement(f[j], "object"));
					info.setKey(getAttributeByElement(f[j], "key"));
					if (info.getInt_type() == DesktopPartsInfo.STATUS_BAR)
						info.setBarcount(getAttributeByElement(f[j], "barcount"));
					resource.addDesktop(info);
				}
			}
		}

		private void doLoadObject(Document doc) throws Exception {
			Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "load-object");
			for (int i=0;i<e.length;i++) {
				LoadObjectInfo info = new LoadObjectInfo();
				info.setName(getAttributeByElement(e[i], "name"));
				info.setObject_string(getAttributeByElement(e[i], "object"));
				resource.addLoadObject(info);
			}
    	}

		private void doMenuChange(Document doc) throws Exception {
			Element [] e = getSpecificElementsByNode(doc.getDocumentElement(), "menu-change");
			for (int i=0;i<e.length;i++) {
				Element [] f = getSpecificElementsByNode(e[i], "logic");
				for (int j=0;j<f.length;j++) {
					MenuChangeInfo info = new MenuChangeInfo();
					info.setAttribute(getAttributeByElement(f[j], "attribute"));
					info.setProperty(getAttributeByElement(f[j], "property"));
					info.setAction(getAttributeByElement(f[j], "action"));
					info.setValue(getAttributeByElement(f[j], "value"));
					info.setKey(getAttributeByElement(f[j], "key"));
					resource.addMenu(info);
				}
			}
		}
	}
}
