package	jp.co.sds_corp.util.pooh.top;

import java.lang.reflect.Method;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Properties;

import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.simple.SimpleMdiFrame;

public abstract class Top {
	protected Properties poohResource = new Properties();
	protected DesktopResource resource;
	protected TopWindow topWin;
	protected static final String MENU_RESOURCE = "jp.co.sds_corp.util.pooh.menu.SubMenuResource"; 

	public Top() {
		try {
			ClassLoader loader = getClass().getClassLoader();
			URL url = loader.getResource("resource/pooh.properties");
			poohResource.load(url.openStream());	
		} catch (Exception e) { e.printStackTrace(); }
	}

	public String getPoohResource() { return poohResource.getProperty("pooh.resource"); }
	public String getPoohPath() { return poohResource.getProperty("pooh.resource.path"); }
	public String getPoohCharset() { return poohResource.getProperty("pooh.resource.charset"); }

    protected void runMDI(SimpleMdiFrame topW) throws Exception {
		LinkedList list = resource.getDesktopPartsLists();
		Iterator iterator = list.iterator();
		while (iterator.hasNext()) {
			DesktopPartsInfo d = (DesktopPartsInfo)iterator.next();
			switch (d.int_type) {
				case DesktopPartsInfo.MENU_BAR:
					topW.createMenu(d.key);
					if (resource.getMenuChangeLists().size()==0) { setMenu(d.key); }
					break;
				case DesktopPartsInfo.STATUS_BAR:
					topW.createStatusBar(d.barcount);
					break;
				case DesktopPartsInfo.TOOL_BAR:
					topW.createToolBar(d.key);
					break;
			}
		}
		menuChange();
		topW.mainLoop();
	}

	protected void menuChange() throws Exception {
		LinkedList list   = resource.getMenuChangeLists();
		Iterator iterator = list.iterator();
		while (iterator.hasNext()) {
			MenuChangeInfo d = (MenuChangeInfo)iterator.next();
			switch (d.int_logic) {
				case MenuChangeInfo.LOGIC_BEAN:
					if (equals(d.attribute, d.property, d.value))
						setMenu(d.key);
					break;
				case MenuChangeInfo.LOGIC_ACTION: break;
				default: System.out.println("Property error!!");
			}
		}
	}

	public Object getMethod(String name) throws Exception {
		try {
			Class [] parameterTypes = new Class[1];
			switch (topWin.getWindowType()) {
				case TopWindow.TYPE_MDI: 
					Class c = ((SimpleMdiFrame)topWin).getClass();
					parameterTypes[0] = Class.forName(MENU_RESOURCE);
					Method method = c.getMethod(name, parameterTypes);
					return method;
				case TopWindow.TYPE_SDI: break;
				case TopWindow.TYPE_DIALOG: break;
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
		return null;
	}	

	public void invoke(Object action) {
		try {
			switch (topWin.getWindowType()) {
				case TopWindow.TYPE_MDI: 
					Method method = (Method)action;
					method.invoke((SimpleMdiFrame)topWin, null);
					break;
				case TopWindow.TYPE_SDI: break;
				case TopWindow.TYPE_DIALOG: break;
			}
		} catch (Exception e) { e.printStackTrace(); }
	}

	public void doSwingEvent(int nEventType, Object [] params) {
		try {
			switch (topWin.getWindowType()) {
				case TopWindow.TYPE_MDI: 
					((SimpleMdiFrame)topWin).doSwingEvent(nEventType, params);
					break;
				case TopWindow.TYPE_SDI: break;
				case TopWindow.TYPE_DIALOG: break;
			}
		} catch (Exception e) { e.printStackTrace(); }
	}

	public void doSwingEvent(int nEventType, String name, Object [] params) {
		try {
			Class [] parameterTypes = new Class[1];
			switch (topWin.getWindowType()) {
				case TopWindow.TYPE_MDI: 
					if (name == null) {
						((SimpleMdiFrame)topWin).doSwingEvent(nEventType, params);
					} else {
						Class c = ((SimpleMdiFrame)topWin).getClass();
						parameterTypes[0] = Class.forName("java.lang.Object");
						Method method = c.getMethod(name, parameterTypes);
						Object [] objects = new Object[1];
						objects[0] = params;
						method.invoke((SimpleMdiFrame)topWin, objects);
					}
					break;
				case TopWindow.TYPE_SDI: break;
				case TopWindow.TYPE_DIALOG: break;
			}
		} catch (Exception e) { e.printStackTrace(); }
	}

	public void invoke(Object action, SubMenuResource sub) {
		try {
			Object [] objects = new Object[1];
			objects[0] = sub;
			switch (topWin.getWindowType()) {
				case TopWindow.TYPE_MDI: 
					Method method = (Method)action;
					method.invoke((SimpleMdiFrame)topWin, objects);
					break;
				case TopWindow.TYPE_SDI: break;
				case TopWindow.TYPE_DIALOG: break;
			}
		} catch (Exception e) { e.printStackTrace(); }
	}

	public abstract boolean equals(String attribute, String name, String value) throws Exception;
	protected void setMenu(String name) throws Exception {
		switch (topWin.getWindowType()) {
			case TopWindow.TYPE_MDI: 
				((SimpleMdiFrame)topWin).activeMenu(name);
				break;
			case TopWindow.TYPE_SDI: break;
			case TopWindow.TYPE_DIALOG: break;
		}
	}
}
