package	jp.co.sds_corp.visualpooh;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;

import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jp.co.sds_corp.util.pooh.menu.RootMenuResource;
import jp.co.sds_corp.util.pooh.menu.SubMenuResource;
import jp.co.sds_corp.util.pooh.popup.PopupAdapter;
import jp.co.sds_corp.util.pooh.simple.SimplePanel;
import jp.co.sds_corp.util.pooh.simple.SimplePopup;

public class PopupResourceEditor extends SimplePanel {
	private PopupResourceTree menuressTree;
	private PopupResourceTree.NodeInfo resourceNodeInfo;
	private SimplePopup popup = null;
	private PopupResourceTree.NodeInfo menuNodeInfo;
	
	public PopupResourceEditor() { super(); }
	public boolean createPane() { 
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(new Insets(5,5,5,5)));
		return true;
	}
	public void initPanel(VisualPoohDoc doc, RootMenuResource _res) {
		menuressTree = new PopupResourceTree();
		menuressTree.createTree(doc, _res);
		add(menuressTree, "Center");

		menuressTree.getTreeView().addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				menuNodeInfo = null; 
				if (e.getClickCount() >= 2)
					menuTreeClicked(e);
			}
		});
		try {
			popup = new SimplePopup(toplevel.getPopupResource("MENURESOURCE_EDITOR"));
		} catch (Exception e) {}
		popup.setActionListener(new PopupAdapter() {
			public Component getComponent() { return menuressTree.getTreeView(); }
			public boolean isShowOK(MouseEvent e) { return showOK(e); }
			public void doPopupMenuSelected(Object object) { popupMenuSelected(object); }
		});
	}

	private PopupResourceTree.NodeInfo getResourceNodeFromLocation(int x, int y) {
		TreePath path = menuressTree.getTreeView().getPathForLocation(x, y);
		if (path == null) {
			return null;
		} else {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			PopupResourceTree.NodeInfo info = (PopupResourceTree.NodeInfo)node.getUserObject();
			return info;
		}
	}

	private void menuTreeClicked(MouseEvent e) {
		PopupResourceTree.NodeInfo info = getResourceNodeFromLocation(e.getX(), e.getY());
		if (info == null)
			return;

		if (info.getLevel() < 2)
			return;

		switch (((SubMenuResource)info.getInfo()).getType()) {
			case SubMenuResource.MENU_NORMAL:
			case SubMenuResource.MENU_RADIO:
			case SubMenuResource.MENU_CHECK:
			case SubMenuResource.MENU_SEPARATOR:
				swingEvent(info, 4);
		}
	}

	public void closed() {}
	public void doSystemBtnClose(WindowEvent e) {}

	private void swingEvent(PopupResourceTree.NodeInfo info, int action) {
		int ev = -1;
		String key = null;
		String md = null;

		switch (info.getLevel()) {
			case 0:
			case 1:
				key = menuressTree.getToplevelTreeName();
				if (action !=6) {
					ev = PoohEventTypes.POPUP_SUB_MENU_ADD + action;
					md = "doRootPopupAction";
				} else {
					ev = PoohEventTypes.POOH_RESOURCE_TREE_POPUP_TEST;
					md = "doPoohTreePopup";
				}
				break;

			case 2:
				ev = PoohEventTypes.POPUP_SUB_MENU_ADD + action;
				key = menuressTree.getToplevelTreeName();
				md = "doSubPopupAction";
				break;
		}

		if (ev != -1) {
			final int event = ev;
			final String method = md;
			final Object [] objects = (key != null) ? new Object[5] : new Object[1];
			objects[0] = new Integer(ev);
			if (objects.length != 1) { 
				objects[1] = key;
				objects[2] = info.getInfo();
				objects[3] = getParentUserObject((DefaultMutableTreeNode)info.getNode());
				objects[4] = info;
			}
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					toplevel.doSwingEvent(event, method, objects);
				}
			});
		}
	}

	public void reload() { 
		menuressTree.reload(); 
	}

	public Object getParentUserObject(DefaultMutableTreeNode parent) {
		return (parent != null) ? parent.getUserObject() : null;
	}

	public void removeLeaf() {
		DefaultMutableTreeNode node; 
		node = (DefaultMutableTreeNode)menuressTree.getTreeView().getLastSelectedPathComponent();
		menuressTree.removeLeaf(node);
	}

	public void addLeaf(int nType, int nLevel, Object object) {
		DefaultMutableTreeNode child = menuressTree.addLeaf(nType, nLevel, object);
		PopupResourceTree.NodeInfo child_info = (PopupResourceTree.NodeInfo)child.getUserObject();

		DefaultMutableTreeNode parent; 
		parent = (DefaultMutableTreeNode)menuressTree.getTreeView().getLastSelectedPathComponent();
		parent.add(child);
		child_info.setNode(parent);
		reload();
	}

	public void insertLeaf(int nType, int nLevel, Object object, int index) {
		DefaultMutableTreeNode child = menuressTree.addLeaf(nType, nLevel, object);
		PopupResourceTree.NodeInfo child_info = (PopupResourceTree.NodeInfo)child.getUserObject();

		DefaultMutableTreeNode my 
			= (DefaultMutableTreeNode)menuressTree.getTreeView().getLastSelectedPathComponent();
		if (my == null) return;
		PopupResourceTree.NodeInfo info = (PopupResourceTree.NodeInfo)my.getUserObject();

		DefaultMutableTreeNode node = (DefaultMutableTreeNode)info.getNode();
		node.insert(child, index);
		child_info.setNode(node);
		reload();
	}

	//------------------------------------------------------------------
	// IPopup interface
	//------------------------------------------------------------------
	public boolean showOK(MouseEvent e) {
		PopupResourceTree.NodeInfo info = getResourceNodeFromLocation(e.getX(), e.getY());
		if (info == null)
			return false;

		popup.setPopupActionSource(info);
		TreePath path = menuressTree.getTreeView().getPathForLocation(e.getX(), e.getY());
		menuressTree.getTreeView().setSelectionPath(path);
		switch (info.getLevel()) {
			case 0:
			case 1: 
				RootMenuResource rmr = (RootMenuResource)info.getInfo();
				popup.setEnabled(false, 0);
				popup.setEnabled((rmr.submenuSize() <= 0) ? true : false, 1); 
				popup.setEnabled(false, 2);
				popup.setEnabled(false, 3);
				popup.setEnabled(false, 4);
				popup.setEnabled(false, 5);
				popup.setEnabled(true, 6);
				break;

			case 2:
				SubMenuResource smr = (SubMenuResource)info.getInfo();
				popup.setEnabled(false, 0);
				if (smr.getType() == SubMenuResource.MENU_MENU || 
					smr.getType() == SubMenuResource.MENU_GROUP ) {
					if (smr.getRootMenuResource().iterator(info.toString()) == null) {
						popup.setEnabled(true, 1);
					} else {
						popup.setEnabled(false, 1);
					}
				} else {
					popup.setEnabled(false, 1);
				}

				popup.setEnabled(true, 2);
				popup.setEnabled(true, 3);
				popup.setEnabled(true, 4);
				popup.setEnabled(true, 5);
				popup.setEnabled(false, 6);
				break;
		}
		return true;
	}
	public void popupMenuSelected(Object object) {
		PopupResourceTree.NodeInfo info = (PopupResourceTree.NodeInfo)popup.getPopupActionSource();
		SubMenuResource smr = (SubMenuResource)object;
		int idx = popup.getComponentIndex((JMenuItem)smr.getThis());
		swingEvent(info, idx);
	}
}
